@@ -520,10 +520,7 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 
 	result->active_bitmap->scan0 = destbuf;
 	result->active_bitmap->reserved = GBD_OWN_SCAN0;
-
-	result->surface = cairo_image_surface_create_for_data ((BYTE*)destbuf, result->cairo_format,
-		result->active_bitmap->width, result->active_bitmap->height, stride);
-
+	
 	*image = result;
 	return Ok;
 
