@@ -42,7 +42,7 @@ static BrushClass vtable = { BrushTypeLinearGradient,
 			     gdip_linear_gradient_clone_brush,
 			     gdip_linear_gradient_destroy };
 
-static void
+static GpStatus
 gdip_linear_gradient_init (GpLineGradient *linear)
 {
 	gdip_brush_init (&linear->base, &vtable);
@@ -53,14 +53,38 @@ gdip_linear_gradient_init (GpLineGradient *linear)
 	linear->angle = 0.0;
 	linear->isAngleScalable = FALSE;
 	linear->presetColors = (InterpolationColors *) GdipAlloc (sizeof (InterpolationColors));
+	if (!linear->presetColors) {
+		return OutOfMemory;
+	}
+
 	linear->presetColors->count = 0;
 	linear->blend = (Blend *) GdipAlloc (sizeof (Blend));
+	if (!linear->blend) {
+		GdipFree (linear->presetColors);
+		return OutOfMemory;
+	}
+
 	linear->blend->count = 1;
 	linear->blend->factors = (float *) GdipAlloc (sizeof (float));
+	if (!linear->blend->factors) {
+		GdipFree (linear->presetColors);
+		GdipFree (linear->blend);
+		return OutOfMemory;
+	}
+
 	linear->blend->positions = (float *) GdipAlloc (sizeof (float));
+	if (!linear->blend->positions) {
+		GdipFree (linear->presetColors);
+		GdipFree (linear->blend->factors);
+		GdipFree (linear->blend);
+		return OutOfMemory;
+	}
+
 	linear->blend->factors [0] = 1.0;
 	linear->blend->positions[0] = 0.0;
 	linear->pattern = NULL;
+
+	return Ok;
 }
 
 static GpLineGradient*
@@ -68,10 +92,14 @@ gdip_linear_gradient_new (void)
 {
 	GpLineGradient *result = (GpLineGradient *) GdipAlloc (sizeof (GpLineGradient));
 
-	if (result)
-		gdip_linear_gradient_init (result);
+	if (result) {
+		if (gdip_linear_gradient_init (result) == Ok)
+			return result;
 
-	return result;
+		GdipFree (result);
+	}
+
+	return NULL;
 }
 
 GpStatus
@@ -83,7 +111,7 @@ gdip_linear_gradient_clone_brush (GpBrush *brush, GpBrush **clonedBrush)
 	if (!brush || !clonedBrush)
 		return InvalidParameter;
 
-	newbrush = (GpLineGradient *) GdipAlloc (sizeof (GpLineGradient));
+	newbrush = gdip_linear_gradient_new ();
 	if (!newbrush)
 		return OutOfMemory;
 
@@ -108,19 +136,22 @@ gdip_linear_gradient_clone_brush (GpBrush *brush, GpBrush **clonedBrush)
 
 	newbrush->presetColors = (InterpolationColors *) GdipAlloc (sizeof (InterpolationColors));
 
-	if (newbrush->presetColors == NULL) 
-		goto NO_PRESET;
+	if (newbrush->presetColors == NULL)
+		goto failure;
 
 	newbrush->presetColors->count = linear->presetColors->count;
 	if (linear->presetColors->count > 0) {
 		newbrush->presetColors->colors = (ARGB *) GdipAlloc (linear->presetColors->count * sizeof (ARGB));
 		if (newbrush->presetColors->colors == NULL) 
-			goto NO_PRESET_COLORS;
+			goto failure;
+
 		memcpy (newbrush->presetColors->colors, linear->presetColors->colors, 
 			linear->presetColors->count * sizeof (ARGB));
+
 		newbrush->presetColors->positions = (float *) GdipAlloc (linear->presetColors->count * sizeof (float));
-		if (newbrush->presetColors->positions == NULL)
-			goto NO_PRESET_POSITIONS;
+		if (!newbrush->presetColors->positions)
+			goto failure;
+
 		memcpy (newbrush->presetColors->positions, linear->presetColors->positions, 
 			linear->presetColors->count * sizeof (float));
 	} else {
@@ -128,43 +159,33 @@ gdip_linear_gradient_clone_brush (GpBrush *brush, GpBrush **clonedBrush)
 	}
 	
 	newbrush->blend = (Blend *) GdipAlloc (sizeof (Blend));
-	if (newbrush->blend == NULL)
-		goto NO_BLEND;
+	if (!newbrush->blend)
+		goto failure;
 
 	newbrush->blend->count = linear->blend->count;
 	if (linear->blend->count > 0) {
 		newbrush->blend->factors = (float *) GdipAlloc (linear->blend->count * sizeof (float));
-		if (newbrush->blend->factors == NULL) 
-			goto NO_BLEND_FACTORS;
+		if (!newbrush->blend->factors)
+			goto failure;
+
 		memcpy (newbrush->blend->factors, linear->blend->factors, linear->blend->count * sizeof (ARGB));
+
 		newbrush->blend->positions = (float *) GdipAlloc (linear->blend->count * sizeof (float));
-		if (newbrush->blend->positions == NULL)
-			goto NO_BLEND_POSITIONS;
+		if (!newbrush->blend->positions)
+			goto failure;
+
 		memcpy (newbrush->blend->positions, linear->blend->positions, linear->blend->count * sizeof (float));
 	} else {
 		memcpy (newbrush->blend, linear->blend, sizeof (Blend));
 	}
 
-	goto SUCCESS;
+	*clonedBrush = (GpBrush *) newbrush;
+	return Ok;
 
- NO_BLEND_POSITIONS:
-	GdipFree (newbrush->blend->factors);
- NO_BLEND_FACTORS:
-	GdipFree (newbrush->blend);
- NO_BLEND:
- NO_PRESET_POSITIONS:
-	GdipFree (newbrush->presetColors->colors);
- NO_PRESET_COLORS:
-	GdipFree (newbrush->presetColors);
- NO_PRESET:
-	GdipFree (newbrush);
+failure:
+	GdipDeleteBrush ((GpBrush *) newbrush);
 	*clonedBrush = NULL;
 	return OutOfMemory;
-
- SUCCESS:
-	*clonedBrush = (GpBrush *) newbrush;
-
-	return Ok;
 }
 
 GpStatus
@@ -283,13 +304,10 @@ create_tile_linear (GpGraphics *graphics, cairo_t *ct, GpLineGradient *linear)
 	GpStatus status;
 	cairo_pattern_t *pat;
 	cairo_matrix_t matrix;
-	GpRectF *rect;
 
 	if (!graphics || !ct || !linear)
 		return InvalidParameter;
 
-	rect = &linear->rectangle;
-
 	gdip_cairo_matrix_copy (&matrix, &linear->matrix);
 	status = GdipInvertMatrix (&matrix);
 	if (status != Ok)
@@ -319,7 +337,6 @@ gdip_linear_gradient_setup (GpGraphics *graphics, GpBrush *brush)
 {
 	cairo_t *ct;
 	GpLineGradient *linear;
-	GpMatrix *product = NULL;
 	GpStatus status = Ok;
 
 	if (!graphics || !brush)
@@ -448,7 +465,10 @@ GdipCreateLineBrushI (GDIPCONST GpPoint *point1, GDIPCONST GpPoint *point2, ARGB
 {
 	GpPointF p1, p2;
 
-	if (!point1 || !point2 || !lineGradient)
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!point1 || !point2 || !lineGradient || wrapMode == WrapModeClamp)	
 		return InvalidParameter;
 
 	p1.X = point1->X;
@@ -467,9 +487,16 @@ GdipCreateLineBrush (GDIPCONST GpPointF *point1, GDIPCONST GpPointF *point2, ARG
 	BOOL xFlipped = FALSE;
 	BOOL yFlipped = FALSE;
 
-	if (!point1 || !point2 || !lineGradient)
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!point1 || !point2 || !lineGradient || wrapMode == WrapModeClamp)
 		return InvalidParameter;
 
+	// For GDI+ compat. A zero-length gradient isn't well-defined.
+	if (point1->X == point2->X && point1->Y == point2->Y)
+		return OutOfMemory;
+
 	linear = gdip_linear_gradient_new ();
 	if (!linear)
 		return OutOfMemory;
@@ -530,7 +557,6 @@ GdipCreateLineBrush (GDIPCONST GpPointF *point1, GDIPCONST GpPointF *point2, ARG
 	gdip_linear_gradient_setup_initial_matrix (linear);
 
 	*lineGradient = linear;
-
 	return Ok;
 }
 
@@ -556,56 +582,77 @@ GdipCreateLineBrushFromRectI (GDIPCONST GpRect *rect, ARGB color1, ARGB color2,
 {
 	GpRectF rectf;
 
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
 	if (!rect || !lineGradient)
 		return InvalidParameter;
 
-	rectf.X = rect->X;
-	rectf.Y = rect->Y;
-	rectf.Width = rect->Width;
-	rectf.Height = rect->Height;
+	if (mode < LinearGradientModeHorizontal || mode > LinearGradientModeBackwardDiagonal) {
+		*lineGradient = NULL;
+		return OutOfMemory;
+	}
 
-	/* FIXME: Check whether angle has to be scalable or not in case of lineargradient mode. */
-	return GdipCreateLineBrushFromRectWithAngle (&rectf, color1, color2,
-		get_angle_from_linear_gradient_mode (mode), FALSE, wrapMode, lineGradient);
+	gdip_RectF_from_Rect (rect, &rectf);
+	return GdipCreateLineBrushFromRect (&rectf, color1, color2, mode, wrapMode, lineGradient);
 }
 
 // coverity[+alloc : arg-*5]
 GpStatus WINGDIPAPI
 GdipCreateLineBrushFromRect (GDIPCONST GpRectF *rect, ARGB color1, ARGB color2, LinearGradientMode mode, GpWrapMode wrapMode, GpLineGradient **lineGradient)
 {
-	/* FIXME: Check whether angle has to be scalable or not in case of lineargradient mode. */
-	return GdipCreateLineBrushFromRectWithAngle (rect, color1, color2,
-		get_angle_from_linear_gradient_mode (mode), FALSE, wrapMode, lineGradient);
-}
-
-// coverity[+alloc : arg-*6]
-GpStatus WINGDIPAPI
-GdipCreateLineBrushFromRectWithAngleI (GDIPCONST GpRect *rect, ARGB color1, ARGB color2, float angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient **lineGradient)
-{
-	GpRectF rectf;
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
 
 	if (!rect || !lineGradient)
 		return InvalidParameter;
 
-	rectf.X = rect->X;
-	rectf.Y = rect->Y;
-	rectf.Width = rect->Width;
-	rectf.Height = rect->Height;
+	if (mode < LinearGradientModeHorizontal || mode > LinearGradientModeBackwardDiagonal) {
+		*lineGradient = NULL;
+		return OutOfMemory;
+	}
 
+	return GdipCreateLineBrushFromRectWithAngle (rect, color1, color2,
+		get_angle_from_linear_gradient_mode (mode), TRUE, wrapMode, lineGradient);
+}
+
+// coverity[+alloc : arg-*6]
+GpStatus WINGDIPAPI
+GdipCreateLineBrushFromRectWithAngleI (GDIPCONST GpRect *rect, ARGB color1, ARGB color2, REAL angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient **lineGradient)
+{
+	GpRectF rectf;
+
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!rect || !lineGradient)
+		return InvalidParameter;
+
+	gdip_RectF_from_Rect (rect, &rectf);
 	return GdipCreateLineBrushFromRectWithAngle (&rectf, color1, color2, angle, 
 						     isAngleScalable, wrapMode, lineGradient);
 }
 
 // coverity[+alloc : arg-*6]
 GpStatus
-GdipCreateLineBrushFromRectWithAngle (GDIPCONST GpRectF *rect, ARGB color1, ARGB color2, float angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient **lineGradient)
+GdipCreateLineBrushFromRectWithAngle (GDIPCONST GpRectF *rect, ARGB color1, ARGB color2, REAL angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient **lineGradient)
 {
 	GpLineGradient *linear;
 
-	if (!rect || !lineGradient || rect->Width == 0.0 || rect->Height == 0.0)
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!rect || !lineGradient || wrapMode == WrapModeClamp)
 		return InvalidParameter;
 
+	if (rect->Width == 0.0 || rect->Height == 0.0) {
+		*lineGradient = NULL;
+		return OutOfMemory;
+	}
+
 	linear = gdip_linear_gradient_new ();
+	if (!linear)
+		return OutOfMemory;
 
 	linear->wrapMode = wrapMode;
 	linear->lineColors [0] = color1;
@@ -622,36 +669,29 @@ GdipCreateLineBrushFromRectWithAngle (GDIPCONST GpRectF *rect, ARGB color1, ARGB
 	gdip_linear_gradient_setup_initial_matrix (linear);
 
 	*lineGradient = linear;
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipGetLineBlendCount (GpLineGradient *brush, int *count)
+GdipGetLineBlendCount (GpLineGradient *brush, INT *count)
 {
 	if (!brush || !count)
 		return InvalidParameter;
 
-	/* If count is less than 1, we are not in a proper state 
-	 * to return blend property. By default, we have one blend
-	 * set. Therefore, count of 1 is acceptible.
-	 */
-	if (brush->blend->count < 1)
-		return WrongState;
-
 	*count = brush->blend->count;
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipSetLineBlend (GpLineGradient *brush, GDIPCONST float *blend, GDIPCONST float *positions, int count)
+GdipSetLineBlend (GpLineGradient *brush, GDIPCONST REAL *blend, GDIPCONST REAL *positions, INT count)
 {
 	float *blendFactors;
 	float *blendPositions;
-	int index;
 
-	if (!brush || !blend || !positions || (count < 2))
+	if (!brush || !blend || !positions || count <= 0)
+		return InvalidParameter;
+
+	if (count >= 2 && (positions[0] != 0.0f || positions[count - 1] != 1.0f))
 		return InvalidParameter;
 
 	if (brush->blend->count != count) {
@@ -675,7 +715,7 @@ GdipSetLineBlend (GpLineGradient *brush, GDIPCONST float *blend, GDIPCONST float
 		brush->blend->positions = blendPositions;
 	}
 
-	for (index = 0; index < count; index++) {
+	for (int index = 0; index < count; index++) {
 		brush->blend->factors [index] = blend [index];
 		brush->blend->positions [index] = positions [index];
 	}
@@ -690,15 +730,17 @@ GdipSetLineBlend (GpLineGradient *brush, GDIPCONST float *blend, GDIPCONST float
 	}
 
 	brush->base.changed = TRUE;
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipGetLineBlend (GpLineGradient *brush, float *blend, float *positions, int count)
+GdipGetLineBlend (GpLineGradient *brush, REAL *blend, REAL *positions, INT count)
 {
-	if (!brush || !blend || !positions || (brush->blend->count != count))
+	if (!brush || !blend || !positions || count <= 0)
 		return InvalidParameter;
+		
+	if (count < brush->blend->count)
+		return InsufficientBuffer;
 
 	/* If count is less than 1, we are not in a proper state 
 	 * to return blend property. By default, we have one blend
@@ -707,8 +749,11 @@ GdipGetLineBlend (GpLineGradient *brush, float *blend, float *positions, int cou
 	if (brush->blend->count < 1)
 		return WrongState;
 	
-	memcpy (blend, brush->blend->factors, count * sizeof (float));
-	memcpy (positions, brush->blend->positions, count * sizeof (float));
+	memcpy (blend, brush->blend->factors, brush->blend->count * sizeof (float));
+
+	// Don't copy anything to positions if the count is one, as positions requires at least 2 values in the array.
+	if (brush->blend->count > 1)
+		memcpy (positions, brush->blend->positions, brush->blend->count * sizeof (float));
 
 	return Ok;
 }
@@ -732,35 +777,26 @@ GdipGetLineGammaCorrection (GpLineGradient *brush, BOOL *useGammaCorrection)
 		return InvalidParameter;
 
 	*useGammaCorrection = brush->gammaCorrection;
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipGetLinePresetBlendCount (GpLineGradient *brush, int *count)
+GdipGetLinePresetBlendCount (GpLineGradient *brush, INT *count)
 {
 	if (!brush || !count)
 		return InvalidParameter;
 
-	/* If count is less than 2, we are not in a proper state 
-	 * to return presetblend property.
-	 */
-	if (brush->presetColors->count < 2)
-		return WrongState;
-
 	*count = brush->presetColors->count;
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipSetLinePresetBlend (GpLineGradient *brush, GDIPCONST ARGB *blend, GDIPCONST float *positions, int count)
+GdipSetLinePresetBlend (GpLineGradient *brush, GDIPCONST ARGB *blend, GDIPCONST REAL *positions, INT count)
 {
 	ARGB *blendColors;
 	float *blendPositions;
-	int index;
 
-	if (!brush || !blend || !positions || (count < 2))
+	if (!brush || !blend || !positions || count < 2 || positions[0] != 0.0f || positions[count - 1] != 1.0f)
 		return InvalidParameter;
 
 	if (brush->presetColors->count != count) {
@@ -784,7 +820,7 @@ GdipSetLinePresetBlend (GpLineGradient *brush, GDIPCONST ARGB *blend, GDIPCONST
 		brush->presetColors->positions = blendPositions;
 	}
 
-	for (index = 0; index < count; index++) {
+	for (int index = 0; index < count; index++) {
 		brush->presetColors->colors [index] = blend [index];
 		brush->presetColors->positions [index] = positions [index];
 	}
@@ -803,11 +839,14 @@ GdipSetLinePresetBlend (GpLineGradient *brush, GDIPCONST ARGB *blend, GDIPCONST
 }
 
 GpStatus WINGDIPAPI
-GdipGetLinePresetBlend (GpLineGradient *brush, ARGB *blend, float *positions, int count)
+GdipGetLinePresetBlend (GpLineGradient *brush, ARGB *blend, REAL *positions, INT count)
 {
-	if (!brush || !blend || !positions || (brush->presetColors->count != count))
+	if (!brush || !blend || !positions || count < 2)
 		return InvalidParameter;
 
+	if (brush->presetColors->count == 0)
+		return GenericError;
+
 	/* If count is less than 2, we are not in a proper state 
 	 * to return presetblend property.
 	 */
@@ -850,11 +889,7 @@ GdipGetLineRectI (GpLineGradient *brush, GpRect *rect)
 	if (!brush || !rect)
 		return InvalidParameter;
 
-	rect->X = (int) brush->rectangle.X;
-	rect->Y = (int) brush->rectangle.Y;
-	rect->Width = (int) brush->rectangle.Width;
-	rect->Height = (int) brush->rectangle.Height;
-
+	gdip_Rect_from_RectF (&brush->rectangle, rect);
 	return Ok;
 }
 
@@ -865,7 +900,6 @@ GdipGetLineRect (GpLineGradient *brush, GpRectF *rect)
 		return InvalidParameter;
 
 	memcpy (rect, &brush->rectangle, sizeof (GpRectF));
-
 	return Ok;
 }
 
@@ -876,22 +910,20 @@ GdipGetLineTransform (GpLineGradient *brush, GpMatrix *matrix)
 		return InvalidParameter;
 
 	gdip_cairo_matrix_copy (matrix, &brush->matrix);
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
 GdipSetLineTransform (GpLineGradient *brush, GDIPCONST GpMatrix *matrix)
 {
-	GpStatus status;
 	BOOL invertible;
 
 	if (!brush || !matrix)
 		return InvalidParameter;
 
 	/* the matrix MUST be invertible to be used */
-	status = GdipIsMatrixInvertible ((GpMatrix*) matrix, &invertible);
-	if (!invertible || (status != Ok))
+	GdipIsMatrixInvertible (matrix, &invertible);
+	if (!invertible)
 		return InvalidParameter;
 
 	gdip_cairo_matrix_copy (&brush->matrix, matrix);
@@ -915,19 +947,22 @@ GdipSetLineWrapMode (GpLineGradient *brush, GpWrapMode wrapMode)
 	if (!brush || (wrapMode == WrapModeClamp))
 		return InvalidParameter;
 
+	if (wrapMode > WrapModeClamp)
+		return Ok;
+
 	brush->wrapMode = wrapMode;
 	brush->base.changed = TRUE;
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipSetLineLinearBlend (GpLineGradient *brush, float focus, float scale)
+GdipSetLineLinearBlend (GpLineGradient *brush, REAL focus, REAL scale)
 {
 	float *blends;
 	float *positions;
 	int count = 3;
 
-	if (!brush)
+	if (!brush || focus < 0 || focus > 1 || scale < 0 || scale > 1)
 		return InvalidParameter;
 
 	if (focus == 0 || focus == 1) {
@@ -993,7 +1028,7 @@ GdipSetLineLinearBlend (GpLineGradient *brush, float focus, float scale)
 }
 
 GpStatus WINGDIPAPI
-GdipSetLineSigmaBlend (GpLineGradient *brush, float focus, float scale)
+GdipSetLineSigmaBlend (GpLineGradient *brush, REAL focus, REAL scale)
 {
 	float *blends;
 	float *positions;
@@ -1013,7 +1048,7 @@ GdipSetLineSigmaBlend (GpLineGradient *brush, float focus, float scale)
 	float curve_top;
 	float curve_height;
 
-	if (!brush)
+	if (!brush || focus < 0 || focus > 1 || scale < 0 || scale > 1)
 		return InvalidParameter;
 
 	if (focus == 0 || focus == 1) {
@@ -1177,22 +1212,23 @@ GdipSetLineSigmaBlend (GpLineGradient *brush, float focus, float scale)
 GpStatus WINGDIPAPI
 GdipMultiplyLineTransform (GpLineGradient *brush, GpMatrix *matrix, GpMatrixOrder order)
 {
-	GpStatus status;
 	BOOL invertible;
 
-	if (!brush || !matrix)
+	if (!brush)
 		return InvalidParameter;
 
+	if (!matrix)
+		return Ok;
+
 	/* the matrix MUST be invertible to be used */
-	status = GdipIsMatrixInvertible ((GpMatrix*) matrix, &invertible);
-	if (!invertible || (status != Ok))
+	GdipIsMatrixInvertible (matrix, &invertible);
+	if (!invertible)
 		return InvalidParameter;
 
-	/* note: error handling is different from GdipMultiplyMatrix */
-	if (order == MatrixOrderAppend)
-		cairo_matrix_multiply (&brush->matrix, &brush->matrix, matrix);
+	if (order == MatrixOrderPrepend)
+		cairo_matrix_multiply (&brush->matrix, matrix, &brush->matrix);
 	else
-		cairo_matrix_multiply (&brush->matrix, matrix, &brush->matrix);        
+		cairo_matrix_multiply (&brush->matrix, &brush->matrix, matrix);       
 
 	brush->base.changed = TRUE;
 	return Ok;
@@ -1211,40 +1247,49 @@ GdipResetLineTransform (GpLineGradient *brush)
 }
 
 GpStatus WINGDIPAPI
-GdipRotateLineTransform (GpLineGradient *brush, float angle, GpMatrixOrder order)
+GdipRotateLineTransform (GpLineGradient *brush, REAL angle, GpMatrixOrder order)
 {
 	GpStatus status;
 
 	if (!brush)
 		return InvalidParameter;
 
-	if ((status = GdipRotateMatrix (&brush->matrix, angle, order)) == Ok)
-		brush->base.changed = TRUE;
-	return status;
+	status = GdipRotateMatrix (&brush->matrix, angle, order);
+	if (status != Ok)
+		return status;
+
+	brush->base.changed = TRUE;
+	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipScaleLineTransform (GpLineGradient *brush, float sx, float sy, GpMatrixOrder order)
+GdipScaleLineTransform (GpLineGradient *brush, REAL sx, REAL sy, GpMatrixOrder order)
 {
 	GpStatus status;
 
 	if (!brush)
 		return InvalidParameter;
 
-	if ((status = GdipScaleMatrix (&brush->matrix, sx, sy, order)) == Ok)
-		brush->base.changed = TRUE;
-	return status;
+	status = GdipScaleMatrix (&brush->matrix, sx, sy, order);
+	if (status != Ok)
+		return status;
+
+	brush->base.changed = TRUE;
+	return Ok;
 }
 
 GpStatus WINGDIPAPI
-GdipTranslateLineTransform (GpLineGradient *brush, float dx, float dy, GpMatrixOrder order)
+GdipTranslateLineTransform (GpLineGradient *brush, REAL dx, REAL dy, GpMatrixOrder order)
 {
 	GpStatus status;
 
 	if (!brush)
 		return InvalidParameter;
 
-	if ((status = GdipTranslateMatrix (&brush->matrix, dx, dy, order)) == Ok)
-		brush->base.changed = TRUE;
-	return status;
+	status = GdipTranslateMatrix (&brush->matrix, dx, dy, order);
+	if (status != Ok)
+		return status;
+
+	brush->base.changed = TRUE;
+	return Ok;
 }
