@@ -1,24 +1,24 @@
 /*
  * pen.c
- * 
+ *
  * Copyright (c) 2003 Alexandre Pigolkine
  * Copyright (C) 2006 Novell, Inc (http://www.novell.com)
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
- * and associated documentation files (the "Software"), to deal in the Software without restriction, 
- * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
- * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
+ * and associated documentation files (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
  * subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in all copies or substantial 
+ *
+ * The above copyright notice and this permission notice shall be included in all copies or substantial
  * portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
- * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
+ * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
  * Authors:
  *   Alexandre Pigolkine(pigolkine@gmx.de)
  *   Duncan Mak (duncan@ximian.com)
@@ -29,18 +29,19 @@
 #include "pen-private.h"
 #include "solidbrush-private.h"
 #include "matrix-private.h"
+#include "general-private.h"
 #include "graphics-private.h"
 #include "customlinecap-private.h"
 
-static void 
+static void
 gdip_pen_init (GpPen *pen)
 {
-        pen->color = 0;
-        pen->brush = 0;
+	pen->color = 0;
+	pen->brush = NULL;
 	pen->own_brush = FALSE;
-        pen->width = 1;
-        pen->miter_limit = 10;
-        pen->line_join = LineJoinMiter;
+	pen->width = 1;
+	pen->miter_limit = 10;
+	pen->line_join = LineJoinMiter;
 	pen->dash_style = DashStyleSolid;
 	pen->line_cap = LineCapFlat;
 	pen->end_cap = LineCapFlat;		/* ignored, Cairo only support a single start/end line cap */
@@ -62,80 +63,83 @@ gdip_pen_init (GpPen *pen)
 static GpPen*
 gdip_pen_new (void)
 {
-        GpPen *result = (GpPen *) GdipAlloc (sizeof (GpPen));
+	GpPen *result = (GpPen *) GdipAlloc (sizeof (GpPen));
 
 	if (result)
 		gdip_pen_init (result);
 
-        return result;
+	return result;
 }
 
 static cairo_line_join_t
 convert_line_join (GpLineJoin join)
 {
-        switch (join) {
+	switch (join) {
 
-        case LineJoinMiter:
-                return CAIRO_LINE_JOIN_MITER;
+	case LineJoinMiter:
+		return CAIRO_LINE_JOIN_MITER;
 
-        case LineJoinBevel:
-                return CAIRO_LINE_JOIN_BEVEL;
+	case LineJoinBevel:
+		return CAIRO_LINE_JOIN_BEVEL;
 
-        case LineJoinRound:
-                return CAIRO_LINE_JOIN_ROUND;
- 
-        case LineJoinMiterClipped:
-        default:
-                return CAIRO_LINE_JOIN_MITER;
-        }
+	case LineJoinRound:
+		return CAIRO_LINE_JOIN_ROUND;
+
+	case LineJoinMiterClipped:
+	default:
+		return CAIRO_LINE_JOIN_MITER;
+	}
 }
 
 static cairo_line_cap_t
 convert_line_cap (GpPen *pen)
 {
-        switch (pen->line_cap) {
+	switch (pen->line_cap) {
 
 	/* HACK - this keeps SWF (mostly) happy with results very similar to GDI+
-	 * (under those specific cases) and also keeps the pen's functionalities 
+	 * (under those specific cases) and also keeps the pen's functionalities
 	 * on par with GDI+
 	 */
-        case LineCapFlat:
+	case LineCapFlat:
 		if (pen->dash_array || (pen->width > 1.0))
-	                return CAIRO_LINE_CAP_BUTT;
+			return CAIRO_LINE_CAP_BUTT;
 		else
-	                return CAIRO_LINE_CAP_SQUARE;
-        
-        case LineCapSquare:
-                return CAIRO_LINE_CAP_SQUARE;
+			return CAIRO_LINE_CAP_SQUARE;
 
-        case LineCapRound:
-                return CAIRO_LINE_CAP_ROUND;                
+	case LineCapSquare:
+		return CAIRO_LINE_CAP_SQUARE;
 
-        case LineCapTriangle:
-        case LineCapNoAnchor:
-        case LineCapSquareAnchor:
-        case LineCapRoundAnchor:
-        case LineCapDiamondAnchor:
-        case LineCapArrowAnchor:
-        case LineCapCustom:
-        default:
-                return CAIRO_LINE_CAP_BUTT;
-        }
+	case LineCapRound:
+		return CAIRO_LINE_CAP_ROUND;
+
+	case LineCapTriangle:
+	case LineCapNoAnchor:
+	case LineCapSquareAnchor:
+	case LineCapRoundAnchor:
+	case LineCapDiamondAnchor:
+	case LineCapArrowAnchor:
+	case LineCapCustom:
+	default:
+		return CAIRO_LINE_CAP_BUTT;
+	}
 }
 
 static double *
 convert_dash_array (float *f, double width, int count)
 {
-        double *retval = GdipAlloc (sizeof (double) * count);
-        int i;
-        for (i = 0; i < count; i++) {
-                retval[i] = (double) f[i] * width;
-        }
+	double *retval = GdipAlloc (sizeof (double) * count);
+	if (!retval)
+		return NULL;
 
-        return retval;
+	int i;
+	for (i = 0; i < count; i++) {
+		retval[i] = (double) f[i] * width;
+	}
+
+	return retval;
 }
 
-GpStatus 
+GpStatus
 gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
 {
 	GpStatus status;
@@ -164,7 +168,7 @@ gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
 		/* *both* X and Y are affected if either is 0 */
 		product.xx = product.yy = 0.0001f;
 	}
-	cairo_set_matrix (graphics->ct, &product);
+	gdip_cairo_set_matrix (graphics, &product);
 
 	/* Don't need to setup, if pen is the same as the cached pen and
 	 * it is not changed. Just comparing pointers may not be sufficient
@@ -185,19 +189,22 @@ gdip_pen_setup (GpGraphics *graphics, GpPen *pen)
 	}
 	cairo_set_line_width (graphics->ct, widthx);
 
-        cairo_set_miter_limit (graphics->ct, (double) pen->miter_limit);
-        cairo_set_line_join (graphics->ct, convert_line_join (pen->line_join));
-        cairo_set_line_cap (graphics->ct, convert_line_cap (pen));
+	cairo_set_miter_limit (graphics->ct, (double) pen->miter_limit);
+	cairo_set_line_join (graphics->ct, convert_line_join (pen->line_join));
+	cairo_set_line_cap (graphics->ct, convert_line_cap (pen));
 
-        if (pen->dash_count > 0) {
-                double *dash_array;
+	if (pen->dash_count > 0) {
+		double *dash_array;
 
-		/* note: pen->width may be different from what was used to 
-		   call cairo_set_line_width, e.g. 0.0 (#78742) */
-                dash_array = convert_dash_array (pen->dash_array, widthx, pen->dash_count);
-                cairo_set_dash (graphics->ct, dash_array, pen->dash_count, pen->dash_offset);
-                GdipFree (dash_array);
-        } else /* Clear the dashes, if set in previous calls */
+		/* note: pen->width may be different from what was used to
+		call cairo_set_line_width, e.g. 0.0 (#78742) */
+		dash_array = convert_dash_array (pen->dash_array, widthx, pen->dash_count);
+		if (!dash_array)
+			return OutOfMemory;
+
+		cairo_set_dash (graphics->ct, dash_array, pen->dash_count, pen->dash_offset);
+		GdipFree (dash_array);
+	} else /* Clear the dashes, if set in previous calls */
 		cairo_set_dash (graphics->ct, NULL, 0, 0);
 
 	/* We are done with using all the changes in the pen. */
@@ -212,7 +219,7 @@ gdip_pen_draw_custom_start_cap (GpGraphics *graphics, GpPen *pen, float x1, floa
 {
 	if (!graphics || !pen)
 		return InvalidParameter;
-	
+
 	if (pen->custom_start_cap) {
 		/* Draw the end cap */
 		gdip_linecap_draw (graphics, pen, pen->custom_start_cap, x1, y1, x2, y2);
@@ -226,7 +233,7 @@ gdip_pen_draw_custom_end_cap (GpGraphics *graphics, GpPen *pen, float x1, float
 {
 	if (!graphics || !pen)
 		return InvalidParameter;
-	
+
 	if (pen->custom_end_cap) {
 		/* Draw the end cap */
 		gdip_linecap_draw (graphics, pen, pen->custom_end_cap, x1, y1, x2, y2);
@@ -236,229 +243,174 @@ gdip_pen_draw_custom_end_cap (GpGraphics *graphics, GpPen *pen, float x1, float
 }
 
 // coverity[+alloc : arg-*3]
-GpStatus 
-GdipCreatePen1 (ARGB argb, float width, GpUnit unit, GpPen **pen)
+GpStatus WINGDIPAPI
+GdipCreatePen1 (ARGB argb, REAL width, GpUnit unit, GpPen **pen)
 {
-        GpStatus s;
-	GpSolidFill *solidBrush = NULL;
+	GpStatus status;
 	GpPen *result;
 
-	if (!pen)
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!pen || unit > UnitCairoPoint || unit == UnitDisplay)
 		return InvalidParameter;
 
-	*pen = result = gdip_pen_new ();
-	if (!result)
-		return OutOfMemory;
-
-	result->color = argb;
-	/* FIXME: do unit conversion when setting width */
-	result->width = width;
-
-	s = GdipCreateSolidFill (argb, &solidBrush);
-	if (s != Ok) {
-		if (solidBrush)
-			GdipDeleteBrush ((GpBrush*) solidBrush);
-		GdipFree (result);
+	result = gdip_pen_new ();
+	if (!result) {
 		*pen = NULL;
-		return s;
+		return OutOfMemory;
 	}
 
-	result->brush = (GpBrush*) solidBrush;
+	result->color = argb;
+	result->width = width;
+	result->unit = unit;
 	result->own_brush = TRUE;
+
+	status = GdipCreateSolidFill (argb, (GpSolidFill **) &result->brush);
+	if (status != Ok) {
+		GdipDeletePen (result);
+		*pen = NULL;
+		return status;
+	}
+
+	*pen = result;
 	return Ok;
 }
 
 // coverity[+alloc : arg-*3]
-GpStatus
-GdipCreatePen2 (GpBrush *brush, float width, GpUnit unit, GpPen **pen)
+GpStatus WINGDIPAPI
+GdipCreatePen2 (GpBrush *brush, REAL width, GpUnit unit, GpPen **pen)
 {
-        ARGB color;
-        GpStatus s;
-        GpBrushType type;
 	GpPen *result;
+	GpStatus status;
+	GpBrushType type;
+	ARGB color;
 
-	if (!brush || !pen)
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!brush || !pen || unit > UnitCairoPoint || unit == UnitDisplay)
 		return InvalidParameter;
 
-        *pen = result = gdip_pen_new ();
-	if (!result)
-		return OutOfMemory;
-
-        /* FIXME: do unit conversion when setting width */
-        result->width = width;
-
-	/* the user supplied brush can be disposed, we must clone it to ensure
-	 * it's valid when we need to set the pen */
-	s = GdipCloneBrush (brush, &result->brush);
-	if (s != Ok) {
-		GdipFree (result);
+	result = gdip_pen_new ();
+	if (!result) {
 		*pen = NULL;
-		return s;
+		return OutOfMemory;
 	}
 
+	result->width = width;
+	result->unit = unit;
 	result->own_brush = TRUE;
 
-        s = GdipGetBrushType (brush, &type);
-	if (s != Ok) {
-		GdipFree (result);
+	/* The user supplied brush can be disposed, we must clone it to ensure
+	 * it's valid when we need to set the pen. */
+	status = GdipCloneBrush (brush, &result->brush);
+	if (status != Ok) {
+		GdipDeletePen (result);
 		*pen = NULL;
-		return s;
+		return status;
 	}
 
-        switch (type) {
+	GdipGetBrushType (brush, &type);
+	if (type == BrushTypeSolidColor) {
+		GdipGetSolidFillColor ((GpSolidFill *) brush, &color);
+		result->color = color;
+	}
 
-        case BrushTypeSolidColor:
-                s = GdipGetSolidFillColor ((GpSolidFill*) brush, &color);
-		if (s != Ok) {
-			GdipFree (result);
-			*pen = NULL;
-			return s;
-		}
-                result->color = color;
-		*pen = result;
-                return Ok;
-
-        case BrushTypeHatchFill:
-        case BrushTypeTextureFill:
-        case BrushTypePathGradient:
-        case BrushTypeLinearGradient:
-		*pen = result;
-		return Ok;
-
-        default:
-		GdipFree (result);
-		*pen = NULL;
-                return GenericError;
-        }
-}
-
-static float *
-clone_dash_array (float *clone, float *array, int size)
-{
-        int i;
-
-        for (i = 0; i < size; i++)
-                clone [i] = array [i];
-
-        return clone;
+	*pen = result;
+	return Ok;
 }
 
 // coverity[+alloc : arg-*1]
-GpStatus 
+GpStatus WINGDIPAPI
 GdipClonePen (GpPen *pen, GpPen **clonepen)
 {
-        GpPen *result;
-        float *dashes;                  /* copy off pen->dash_array */
-        float *compound_array = NULL;   /* copy off pen->compound_array */
-	GpCustomLineCap *custom_start_cap = NULL;
-	GpCustomLineCap *custom_end_cap = NULL;
+	GpPen *result;
 
 	if (!pen || !clonepen)
 		return InvalidParameter;
 
-	/* we make a copy of dash array only if it is owned by pen, i.e. it is not
-	 * our global array.
-	 */
-
-	if (pen->dash_count > 0 && pen->own_dash_array) {
-		dashes = (float *) GdipAlloc (pen->dash_count * sizeof (float));
-		if (!dashes)
-			return OutOfMemory;
-		clone_dash_array (dashes, pen->dash_array, pen->dash_count);
-	} else
-		dashes = pen->dash_array;
-
-	if (pen->compound_count > 0) {
-		compound_array = (float *) GdipAlloc (pen->compound_count * sizeof (float));
-		if (compound_array == NULL) {
-			if (pen->dash_count > 0)
-				GdipFree (dashes);
-			return OutOfMemory;
-		}
-		clone_dash_array (compound_array, pen->compound_array, pen->compound_count);
-	}
-
-	if (pen->custom_start_cap != NULL) {
-		GpStatus status = GdipCloneCustomLineCap (pen->custom_start_cap, &custom_start_cap);
-		if (status != Ok) {
-			if (custom_start_cap)
-				GdipDeleteCustomLineCap (custom_start_cap);
-			if (compound_array != NULL)
-				GdipFree (compound_array);
-			if (pen->dash_count > 0)
-				GdipFree (dashes);
-			return OutOfMemory;
-		}
-	}
-
-	if (pen->custom_end_cap != NULL) {
-		GpStatus status = GdipCloneCustomLineCap (pen->custom_end_cap, &custom_end_cap);
-		if (status != Ok) {
-			if (custom_end_cap)
-				GdipDeleteCustomLineCap (custom_end_cap);
-			if (custom_start_cap)
-				GdipDeleteCustomLineCap (custom_start_cap);
-			if (compound_array != NULL)
-				GdipFree (compound_array);
-			if (pen->dash_count > 0)
-				GdipFree (dashes);
-			return OutOfMemory;
-		}
-	}
-
 	result = gdip_pen_new ();
-	if (result == NULL) {
-		if (pen->dash_count > 0)
-			GdipFree (dashes);
-		if (pen->compound_count > 0)
-			GdipFree (compound_array);
-
+	if (!result) {
+		*clonepen = NULL;
 		return OutOfMemory;
 	}
 
+	result->own_brush = pen->own_brush;
+	result->color = pen->color;
+	result->width = pen->width;
+	result->miter_limit = pen->miter_limit;
+	result->line_join = pen->line_join;
+	result->dash_style = pen->dash_style;
+	result->line_cap = pen->line_cap;
+	result->end_cap = pen->end_cap;
+	result->mode = pen->mode;
+	result->dash_offset = pen->dash_offset;
+	result->dash_count = pen->dash_count;
+	result->own_dash_array = pen->own_dash_array;
+	result->compound_count = pen->compound_count;
+	result->unit = pen->unit;
+	gdip_cairo_matrix_copy (&result->matrix, &pen->matrix);
+	result->changed = pen->changed;
+
+	/* Make a copy of dash array only if it is owned by the pen - i.e. it is not
+	 * a global array. */
+	if (pen->dash_count > 0 && pen->own_dash_array) {
+		result->dash_array = (float *) GdipAlloc (pen->dash_count * sizeof (float));
+		if (!result->dash_array)
+			goto error;
+
+		memcpy (result->dash_array, pen->dash_array, pen->dash_count * sizeof (float));
+	}
+	else
+		result->dash_array = pen->dash_array;
+
+	if (pen->compound_count > 0) {
+		result->compound_array = (float *) GdipAlloc (pen->compound_count * sizeof (float));
+		if (!result->compound_array)
+			goto error;
+
+		memcpy (result->compound_array, pen->compound_array, pen->compound_count * sizeof (float));
+	}
+
+	if (pen->custom_start_cap) {
+		GpStatus status = GdipCloneCustomLineCap (pen->custom_start_cap, &result->custom_start_cap);
+		if (status != Ok)
+			goto error;
+	}
+
+	if (pen->custom_end_cap) {
+		GpStatus status = GdipCloneCustomLineCap (pen->custom_end_cap, &result->custom_end_cap);
+		if (status != Ok)
+			goto error;
+	}
+
 	if (pen->own_brush) {
-		GpSolidFill* solidBrush;
-		GpSolidFill* oldbrush = (GpSolidFill*)pen->brush;
-		GdipCreateSolidFill (oldbrush->color, &solidBrush);
-		result->brush = (GpBrush*) solidBrush;
+		GpSolidFill *oldBrush = (GpSolidFill *) pen->brush;
+		GpStatus status = GdipCreateSolidFill (oldBrush->color, (GpSolidFill **) &result->brush);
+		if (status != Ok)
+			goto error;
 	} else {
 		result->brush = pen->brush;
 	}
 
-        result->own_brush = pen->own_brush;
-	result->color = pen->color;
-        result->width = pen->width;
-        result->miter_limit = pen->miter_limit;
-        result->line_join = pen->line_join;
-	result->dash_style = pen->dash_style;
-        result->line_cap = pen->line_cap;
-	result->end_cap = pen->end_cap;
-	result->mode = pen->mode;
-        result->dash_offset = pen->dash_offset;
-	result->dash_count = pen->dash_count;
-	result->own_dash_array = pen->own_dash_array;
-	result->dash_array = dashes;
-	result->compound_count = pen->compound_count;
-	result->compound_array = compound_array;
-	result->unit = pen->unit;
-	gdip_cairo_matrix_copy (&result->matrix, &pen->matrix);
-	result->changed = pen->changed;
-	result->custom_start_cap = custom_start_cap;
-	result->custom_end_cap = custom_end_cap;
+	*clonepen = result;
+	return Ok;
 
-        *clonepen = result;
+error:
+	GdipDeletePen (result);
+	*clonepen = NULL;
+	return OutOfMemory;
+}
 
-        return Ok;
-}       
-
-GpStatus 
+GpStatus WINGDIPAPI
 GdipDeletePen (GpPen *pen)
 {
 	if (!pen)
 		return InvalidParameter;
 
-        if (pen->dash_count != 0 && pen->own_dash_array) {
-                GdipFree (pen->dash_array);
+	if (pen->dash_count != 0 && pen->own_dash_array) {
+		GdipFree (pen->dash_array);
 		pen->dash_count = 0;
 		pen->dash_array = NULL;
 	}
@@ -468,8 +420,8 @@ GdipDeletePen (GpPen *pen)
 		pen->brush = NULL;
 	}
 
-        if (pen->compound_count != 0) {
-                GdipFree (pen->compound_array);
+	if (pen->compound_count != 0) {
+		GdipFree (pen->compound_array);
 		pen->compound_array = NULL;
 		pen->compound_count = 0;
 	}
@@ -484,66 +436,59 @@ GdipDeletePen (GpPen *pen)
 		pen->custom_end_cap = NULL;
 	}
 
-        GdipFree (pen);
+	GdipFree (pen);
 	return Ok;
 }
 
-GpStatus
-GdipSetPenWidth (GpPen *pen, float width)
+GpStatus WINGDIPAPI
+GdipSetPenWidth (GpPen *pen, REAL width)
 {
 	if (!pen)
 		return InvalidParameter;
 
-        pen->width = width;
-	pen->changed = TRUE;
-        return Ok;
+	if (pen->width != width) {
+		pen->width = width;
+		pen->changed = TRUE;
+	}
+
+	return Ok;
 }
 
-GpStatus
-GdipGetPenWidth (GpPen *pen, float *width)
+GpStatus WINGDIPAPI
+GdipGetPenWidth (GpPen *pen, REAL *width)
 {
 	if (!pen || !width)
 		return InvalidParameter;
 
-        *width = pen->width;
-        return Ok;
+	*width = pen->width;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenBrushFill (GpPen *pen, GpBrush *brush)
 {
-        GpStatus s;
-        ARGB color;
-        GpBrushType type;
+	GpBrushType type;
 
 	if (!pen || !brush)
 		return InvalidParameter;
 
-        s = GdipGetBrushType (brush, &type);
-
-	if (s != Ok)
-		return s;
-
-        if (type == BrushTypeSolidColor) {
-        	s = GdipGetSolidFillColor ((GpSolidFill*) brush, &color);
-	        if (s != Ok)
-	                return s;
-        	pen->color = color;
-        }
+	GdipGetBrushType (brush, &type);
+	if (type == BrushTypeSolidColor)
+		GdipGetSolidFillColor ((GpSolidFill *) brush, &pen->color);
 	else
 		pen->color = 0;
 
 	if (pen->own_brush && pen->brush)
 		GdipDeleteBrush (pen->brush);
 
-        pen->brush = brush;
+	pen->brush = brush;
 	pen->changed = TRUE;
 	pen->own_brush = FALSE;
 
-        return Ok;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenBrushFill (GpPen *pen, GpBrush **brush)
 {
 	if (!pen)
@@ -552,47 +497,66 @@ GdipGetPenBrushFill (GpPen *pen, GpBrush **brush)
 	return GdipCloneBrush (pen->brush, brush);
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenFillType (GpPen *pen, GpPenType *type)
 {
 	if (!pen || !type)
 		return InvalidParameter;
 
-	if (pen->brush != NULL)
+	if (pen->brush)
 		return GdipGetBrushType (pen->brush, (GpBrushType *) type);
 
 	*type = PenTypeSolidColor;
-
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenColor (GpPen *pen, ARGB argb)
 {
+	GpStatus status;
+	GpSolidFill *brush;
+	GpBrushType type;
+
 	if (!pen)
 		return InvalidParameter;
 
-	pen->changed = pen->changed ? TRUE : (pen->color != argb);
-	pen->color = argb;
+	GdipGetBrushType (pen->brush, &type);
+	if (type == BrushTypeSolidColor) {
+		ARGB brushColor;
+		GdipGetSolidFillColor ((GpSolidFill *) pen->brush, &brushColor);
 
-	if (pen->changed && pen->brush && (pen->brush->vtable->type == BrushTypeSolidColor))
-		return GdipSetSolidFillColor ((GpSolidFill*)pen->brush, argb);
+		// Nothing to do.
+		if (brushColor == argb)
+			return Ok;
+	}
+
+	// Override the brush and set it to a solid fill.
+	status = GdipCreateSolidFill (argb, &brush);
+	if (status != Ok)
+		return status;
+
+	pen->color = argb;
+	pen->brush = (GpBrush *) brush;
+	pen->changed = TRUE;
 
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenColor (GpPen *pen, ARGB *argb)
 {
 	if (!pen || !argb)
 		return InvalidParameter;
 
-        *argb = pen->color;
-        return Ok;
+	if(pen->brush->vtable->type != BrushTypeSolidColor)
+		return InvalidParameter;
+
+	*argb = pen->color;
+	return Ok;
 }
 
-GpStatus
-GdipSetPenMiterLimit (GpPen *pen, float miterLimit)
+GpStatus WINGDIPAPI
+GdipSetPenMiterLimit (GpPen *pen, REAL miterLimit)
 {
 	if (!pen)
 		return InvalidParameter;
@@ -600,43 +564,49 @@ GdipSetPenMiterLimit (GpPen *pen, float miterLimit)
 	if (miterLimit < 1.0f)
 		miterLimit = 1.0f;
 
-	pen->changed = pen->changed ? TRUE : (pen->miter_limit != miterLimit);
-        pen->miter_limit = miterLimit;
-        return Ok;
+	if (pen->miter_limit != miterLimit) {
+		pen->miter_limit = miterLimit;
+		pen->changed = TRUE;
+	}
+
+	return Ok;
 }
 
-GpStatus
-GdipGetPenMiterLimit (GpPen *pen, float *miterLimit)
+GpStatus WINGDIPAPI
+GdipGetPenMiterLimit (GpPen *pen, REAL *miterLimit)
 {
 	if (!pen || !miterLimit)
 		return InvalidParameter;
 
-        *miterLimit = pen->miter_limit;
-        return Ok;
+	*miterLimit = pen->miter_limit;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenLineJoin (GpPen *pen, GpLineJoin lineJoin)
 {
 	if (!pen)
 		return InvalidParameter;
 
-	pen->changed = pen->changed ? TRUE : (pen->line_join != lineJoin);
-        pen->line_join = lineJoin;
-        return Ok;
+	if (pen->line_join != lineJoin) {
+		pen->line_join = lineJoin;
+		pen->changed = TRUE;
+	}
+
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenLineJoin (GpPen *pen, GpLineJoin *lineJoin)
 {
 	if (!pen || !lineJoin)
 		return InvalidParameter;
 
-        *lineJoin = pen->line_join;
-        return Ok;
+	*lineJoin = pen->line_join;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenLineCap197819 (GpPen *pen, GpLineCap startCap, GpLineCap endCap, GpDashCap dashCap)
 {
 	if (!pen)
@@ -657,120 +627,126 @@ GdipSetPenLineCap197819 (GpPen *pen, GpLineCap startCap, GpLineCap endCap, GpDas
 		pen->dash_cap = DashCapFlat; /* default */
 
 	pen->changed = TRUE;
-        return Ok;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenMode (GpPen *pen, GpPenAlignment penMode)
 {
 	if (!pen)
 		return InvalidParameter;
+	if (penMode == PenAlignmentInset && pen->compound_count > 0)
+		return NotImplemented;
+
+	if (pen->mode != penMode) {
+		pen->mode = penMode;
+		pen->changed = TRUE;
+	}
 
-	pen->changed = pen->changed ? TRUE : (penMode != pen->mode);
-	pen->mode = penMode;
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenMode (GpPen *pen, GpPenAlignment *penMode)
 {
 	if (!pen || !penMode)
 		return InvalidParameter;
 
-        *penMode = pen->mode;
-        return Ok;
+	*penMode = pen->mode;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenUnit (GpPen *pen, GpUnit *unit)
 {
 	if (!pen || !unit)
 		return InvalidParameter;
 
-        *unit = pen->unit;
-        return Ok;
+	*unit = pen->unit;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenUnit (GpPen *pen, GpUnit unit)
 {
-	if (!pen)
+	if (!pen || unit < UnitWorld || unit > UnitCairoPoint || unit == UnitDisplay)
 		return InvalidParameter;
 
-        pen->unit = unit;
-	pen->changed = TRUE;
-        return Ok;
+	if (unit != pen->unit) {
+		pen->unit = unit;
+		pen->changed = TRUE;
+	}
+
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenTransform (GpPen *pen, GpMatrix *matrix)
 {
-	GpStatus status;
 	BOOL invertible;
 
 	if (!pen || !matrix)
 		return InvalidParameter;
 
 	/* the matrix MUST be invertible to be used */
-	status = GdipIsMatrixInvertible ((GpMatrix*) matrix, &invertible);
-	if (!invertible || (status != Ok))
+	GdipIsMatrixInvertible (matrix, &invertible);
+	if (!invertible)
 		return InvalidParameter;
 
 	gdip_cairo_matrix_copy (&pen->matrix, matrix);
 	pen->changed = TRUE;
-        return Ok;
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenTransform (GpPen *pen, GpMatrix *matrix)
 {
 	if (!pen || !matrix)
 		return InvalidParameter;
 
-	gdip_cairo_matrix_copy(matrix, &pen->matrix);
-        return Ok;
+	gdip_cairo_matrix_copy (matrix, &pen->matrix);
+	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipResetPenTransform (GpPen *pen)
 {
 	if (!pen)
 		return InvalidParameter;
 
 	cairo_matrix_init_identity (&pen->matrix);
-
 	pen->changed = TRUE;
 
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipMultiplyPenTransform (GpPen *pen, GpMatrix *matrix, GpMatrixOrder order)
 {
-	GpStatus status;
 	BOOL invertible;
 
-	if (!pen || !matrix)
+	if (!pen)
 		return InvalidParameter;
 
+	if (!matrix)
+		return Ok;
+
 	/* the matrix MUST be invertible to be used */
-	status = GdipIsMatrixInvertible ((GpMatrix*) matrix, &invertible);
-	if (!invertible || (status != Ok))
+	GdipIsMatrixInvertible (matrix, &invertible);
+	if (!invertible)
 		return InvalidParameter;
 
-	/* invalid GpMatrixOrder values are computed as MatrixOrderAppend (i.e. no error) */
-	if (order != MatrixOrderPrepend)
-		order = MatrixOrderAppend;
+	if (order == MatrixOrderPrepend)
+		cairo_matrix_multiply (&pen->matrix, matrix, &pen->matrix);
+	else
+		cairo_matrix_multiply (&pen->matrix, &pen->matrix, matrix);
 
-	status = GdipMultiplyMatrix (&pen->matrix, matrix, order);
-	if (status == Ok)
-		pen->changed = TRUE;
-
-	return status;
+	pen->changed = TRUE;
+	return Ok;
 }
 
-GpStatus
-GdipTranslatePenTransform (GpPen *pen, float dx, float dy, GpMatrixOrder order)
+GpStatus WINGDIPAPI
+GdipTranslatePenTransform (GpPen *pen, REAL dx, REAL dy, GpMatrixOrder order)
 {
 	GpStatus status;
 
@@ -778,31 +754,31 @@ GdipTranslatePenTransform (GpPen *pen, float dx, float dy, GpMatrixOrder order)
 		return InvalidParameter;
 
 	status = GdipTranslateMatrix (&pen->matrix, dx, dy, order);
+	if (status != Ok)
+		return status;
 
-	if (status == Ok)
-		pen->changed = TRUE;
-
-	return status;
+	pen->changed = TRUE;
+	return Ok;
 }
 
-GpStatus
-GdipScalePenTransform (GpPen *pen, float sx, float sy, GpMatrixOrder order)
+GpStatus WINGDIPAPI
+GdipScalePenTransform (GpPen *pen, REAL sx, REAL sy, GpMatrixOrder order)
 {
 	GpStatus status;
 
 	if (!pen)
 		return InvalidParameter;
 
-        status = GdipScaleMatrix (&pen->matrix, sx, sy, order);
+	status = GdipScaleMatrix (&pen->matrix, sx, sy, order);
+	if (status != Ok)
+		return status;
 
-	if (status == Ok)
-		pen->changed = TRUE;
-
-	return status;
+	pen->changed = TRUE;
+	return Ok;
 }
 
-GpStatus
-GdipRotatePenTransform (GpPen *pen, float angle, GpMatrixOrder order)
+GpStatus WINGDIPAPI
+GdipRotatePenTransform (GpPen *pen, REAL angle, GpMatrixOrder order)
 {
 	GpStatus status;
 
@@ -810,21 +786,21 @@ GdipRotatePenTransform (GpPen *pen, float angle, GpMatrixOrder order)
 		return InvalidParameter;
 
 	status = GdipRotateMatrix (&pen->matrix, angle, order);
+	if (status != Ok)
+		return status;
 
-	if (status == Ok)
-		pen->changed = TRUE;
-
-	return status;
+	pen->changed = TRUE;
+	return Ok;
 }
 
-GpStatus
-GdipGetPenDashStyle (GpPen *pen, GpDashStyle *dashStyle)
+GpStatus WINGDIPAPI
+GdipGetPenDashStyle (GpPen *pen, GpDashStyle *dashstyle)
 {
-	if (!pen || !dashStyle)
+	if (!pen || !dashstyle)
 		return InvalidParameter;
 
-        *dashStyle = pen->dash_style;
-        return Ok;
+	*dashstyle = pen->dash_style;
+	return Ok;
 }
 
 static float Dot []  = { 1.0, 1.0 };
@@ -832,13 +808,13 @@ static float Dash []  = { 3.0, 1.0 };
 static float DashDot [] = { 3.0, 1.0, 1.0, 1.0 };
 static float DashDotDot [] = { 3.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
 
-GpStatus
-GdipSetPenDashStyle (GpPen *pen, GpDashStyle dashStyle)
+GpStatus WINGDIPAPI
+GdipSetPenDashStyle (GpPen *pen, GpDashStyle dashstyle)
 {
 	if (!pen)
 		return InvalidParameter;
 
-	switch (dashStyle) {
+	switch (dashstyle) {
 	case DashStyleSolid:
 		pen->dash_array = NULL;
 		pen->dash_count = 0;
@@ -870,79 +846,90 @@ GdipSetPenDashStyle (GpPen *pen, GpDashStyle dashStyle)
 		break;
 
 	default:
-		return GenericError;
-        }
+		/* GDI+ does nothing if the dash style is invalid */
+		return Ok;
+	}
 
-	pen->dash_style = dashStyle;
+	pen->dash_style = dashstyle;
 	pen->changed = TRUE;
 	return Ok;
 }
 
-GpStatus
-GdipGetPenDashOffset (GpPen *pen, float *offset)
+GpStatus WINGDIPAPI
+GdipGetPenDashOffset (GpPen *pen, REAL *offset)
 {
 	if (!pen || !offset)
 		return InvalidParameter;
 
-        *offset = pen->dash_offset;
-        return Ok;
+	*offset = pen->dash_offset;
+	return Ok;
 }
 
-GpStatus
-GdipSetPenDashOffset (GpPen *pen, float offset)
+GpStatus WINGDIPAPI
+GdipSetPenDashOffset (GpPen *pen, REAL offset)
 {
 	if (!pen)
 		return InvalidParameter;
 
-	pen->dash_offset = offset;
-	pen->changed = TRUE;
-        return Ok;
-}
-
-GpStatus
-GdipGetPenDashCount (GpPen *pen, int *count)
-{
-	if (!pen || !count)
-		return InvalidParameter;
-
-        *count = pen->dash_count;
-        return Ok;
-}
-
-/*
- * This is the DashPattern property in Pen.
- */
-GpStatus
-GdipGetPenDashArray (GpPen *pen, float *dash, int count)
-{
-	if (!pen || !dash)
-		return InvalidParameter;
-	if (count == 0)
-		return OutOfMemory;
-	if (count != pen->dash_count)
-		return InvalidParameter;
-
-	memcpy (dash, pen->dash_array, count * sizeof (float));
+	if (pen->dash_offset != offset) {
+		pen->dash_offset = offset;
+		pen->changed = TRUE;
+	}
 
 	return Ok;
 }
 
-GpStatus
-GdipSetPenDashArray (GpPen *pen, GDIPCONST float *dash, int count)
+GpStatus WINGDIPAPI
+GdipGetPenDashCount (GpPen *pen, INT *count)
+{
+	if (!pen || !count)
+		return InvalidParameter;
+
+	*count = pen->dash_count;
+	return Ok;
+}
+
+GpStatus WINGDIPAPI
+GdipGetPenDashArray (GpPen *pen, REAL *dash, INT count)
+{
+	if (!pen || !dash || count > pen->dash_count)
+		return InvalidParameter;
+	if (!pen->dash_array)
+		return OutOfMemory;
+	if (count < 0)
+		return OutOfMemory;
+
+	memcpy (dash, pen->dash_array, count * sizeof (float));
+	return Ok;
+}
+
+GpStatus WINGDIPAPI
+GdipSetPenDashArray (GpPen *pen, GDIPCONST REAL *dash, INT count)
 {
 	float *dash_array;
+	REAL sum = 0;
 
 	if (!pen || !dash || (count <= 0))
 		return InvalidParameter;
 
-	if (pen->dash_count != count || pen->own_dash_array == FALSE) {
+	for(int i = 0; i < count; i++) {
+		sum += dash[i];
+		if(dash[i] < 0.0)
+			return InvalidParameter;
+	}
+
+	if (sum == 0.0)
+		return InvalidParameter;
+
+	if (pen->dash_count != count || !pen->own_dash_array) {
 		dash_array = (float *) GdipAlloc (count * sizeof (float));
 		if (!dash_array)
 			return OutOfMemory;
 
 		/* free the existing values, if we own them */
-		if (pen->dash_count != 0 && pen->own_dash_array == TRUE)
+		if (pen->dash_count != 0 && pen->own_dash_array)
 			GdipFree (pen->dash_array);
+
 		pen->dash_array = dash_array;
 		pen->dash_count = count;
 		pen->own_dash_array = TRUE;
@@ -958,8 +945,8 @@ GdipSetPenDashArray (GpPen *pen, GDIPCONST float *dash, int count)
 /*
  * MonoTODO: Find out what the difference is between CompoundArray and DashArray
  */
-GpStatus
-GdipGetPenCompoundCount (GpPen *pen, int *count)
+GpStatus WINGDIPAPI
+GdipGetPenCompoundCount (GpPen *pen, INT *count)
 {
 	if (!pen || !count)
 		return InvalidParameter;
@@ -968,13 +955,25 @@ GdipGetPenCompoundCount (GpPen *pen, int *count)
 	return Ok;
 }
 
-GpStatus
-GdipSetPenCompoundArray (GpPen *pen, const float *compound, int count)
+GpStatus WINGDIPAPI
+GdipSetPenCompoundArray (GpPen *pen, GDIPCONST REAL *compound, INT count)
 {
+	float minValue = 0.0f;
 	float *compound_array;
 
-	if (!pen || !compound || (count <= 0))
+	if (!pen || !compound || count <= 0 || count % 2 == 1)
 		return InvalidParameter;
+	if (pen->mode == PenAlignmentInset)
+		return NotImplemented;
+
+	/* All values have to be in ascending order and be between 0 and 1. */
+	for (int i = 0; i < count; i++) {
+		float value = compound[i];
+		if (value < minValue || value > 1.0f)
+			return InvalidParameter;
+
+		minValue = value;
+	}
 
 	if (pen->compound_count != count) {
 		compound_array = (float *) GdipAlloc (count * sizeof (float));
@@ -982,52 +981,58 @@ GdipSetPenCompoundArray (GpPen *pen, const float *compound, int count)
 			return OutOfMemory;
 
 		/* free the existing values */
-		if (pen->compound_count != 0)
+		if (pen->compound_count)
 			GdipFree (pen->compound_array);
+
 		pen->compound_array = compound_array;
 		pen->compound_count = count;
 	}
 
 	memcpy (pen->compound_array, compound, pen->compound_count * sizeof (float));
-
 	return Ok;
 }
 
-GpStatus
-GdipGetPenCompoundArray (GpPen *pen, float *compound, int count)
+GpStatus WINGDIPAPI
+GdipGetPenCompoundArray (GpPen *pen, REAL *compound, INT count)
 {
-	if (!pen || !compound || (count != pen->compound_count))
+	if (!pen || !compound || count > pen->compound_count)
 		return InvalidParameter;
 
+	/* GDI+ does nothing if count <= 0. */
+	if (count <= 0)
+		return Ok;
+
 	memcpy (compound, pen->compound_array, count * sizeof (float));
 
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenStartCap (GpPen *pen, GpLineCap startCap)
 {
 	if (!pen)
 		return InvalidParameter;
 
+	GdipDeleteCustomLineCap (pen->custom_start_cap);
+	pen->custom_start_cap = NULL;
+
 	pen->line_cap = startCap;
 	pen->changed = TRUE;
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenStartCap (GpPen *pen, GpLineCap *startCap)
 {
 	if (!pen || !startCap)
 		return InvalidParameter;
 
 	*startCap = pen->line_cap;
-
 	return Ok;
 }
 
 /* MonoTODO - ignored (always same as start cap) */
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenEndCap (GpPen *pen, GpLineCap endCap)
 {
 	if (!pen)
@@ -1037,6 +1042,10 @@ GdipSetPenEndCap (GpPen *pen, GpLineCap endCap)
 	 * This is currently ignored, as Cairo does not support
 	 * having a different start and end Cap
 	 */
+
+	GdipDeleteCustomLineCap (pen->custom_end_cap);
+	pen->custom_end_cap = NULL;
+
 	pen->end_cap = endCap;
 	pen->changed = TRUE;
 
@@ -1044,30 +1053,34 @@ GdipSetPenEndCap (GpPen *pen, GpLineCap endCap)
 }
 
 /* MonoTODO - ignored (always same as start cap) */
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenEndCap (GpPen *pen, GpLineCap *endCap)
 {
 	if (!pen || !endCap)
 		return InvalidParameter;
 
 	*endCap = pen->end_cap;
-
 	return Ok;
 }
 
 /* MonoTODO - ignored */
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenDashCap197819 (GpPen *pen, GpDashCap dashCap)
 {
 	if (!pen)
 		return InvalidParameter;
 
-	pen->dash_cap = dashCap;
+	/* Any invalid value is changed to DashCapFlat. */
+	if (dashCap == DashCapRound || dashCap == DashCapTriangle)
+		pen->dash_cap = dashCap;
+	else
+		pen->dash_cap = DashCapFlat;
+
 	return Ok;
 }
 
 /* MonoTODO - ignored */
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenDashCap197819 (GpPen *pen, GpDashCap *dashCap)
 {
 	if (!pen || !dashCap)
@@ -1077,38 +1090,66 @@ GdipGetPenDashCap197819 (GpPen *pen, GpDashCap *dashCap)
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenCustomStartCap (GpPen *pen, GpCustomLineCap *customCap)
 {
+	GpStatus status;
+	GpCustomLineCap *clonedCap;
+
 	if (!pen)
 		return InvalidParameter;
-	
-	return GdipCloneCustomLineCap (customCap, &pen->custom_start_cap);
+
+	status = GdipCloneCustomLineCap (customCap, &clonedCap);
+	if (status == Ok) {
+		pen->custom_start_cap = clonedCap;
+		pen->line_cap = LineCapCustom;
+	}
+
+	return status;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenCustomStartCap (GpPen *pen, GpCustomLineCap **customCap)
 {
 	if (!pen || !customCap)
 		return InvalidParameter;
 
+	if (!pen->custom_start_cap) {
+		*customCap = NULL;
+		return Ok;
+	}
+
 	return GdipCloneCustomLineCap (pen->custom_start_cap, customCap);
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetPenCustomEndCap (GpPen *pen, GpCustomLineCap *customCap)
 {
+	GpStatus status;
+	GpCustomLineCap *clonedCap;
+
 	if (!pen)
 		return InvalidParameter;
-	
-	return GdipCloneCustomLineCap (customCap, &pen->custom_end_cap);
+
+	status = GdipCloneCustomLineCap (customCap, &clonedCap);
+	if (status == Ok) {
+		pen->custom_end_cap = clonedCap;
+		pen->end_cap = LineCapCustom;
+	}
+
+	return status;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetPenCustomEndCap (GpPen *pen, GpCustomLineCap **customCap)
 {
 	if (!pen || !customCap)
 		return InvalidParameter;
 
+	if (!pen->custom_end_cap) {
+		*customCap = NULL;
+		return Ok;
+	}
+
 	return GdipCloneCustomLineCap (pen->custom_end_cap, customCap);
 }
