@@ -23,15 +23,23 @@
 #ifndef _REGION_PATH_TREE_H_
 #define _REGION_PATH_TREE_H_
 
-#include "gdip.h"
+#include "gdiplus-private.h"
+#include "matrix-private.h"
 
 /* internal (private) API for regions path trees */
 
 #define REGION_TAG_PATH		1
 #define REGION_TAG_TREE		2
 
+typedef struct GpPathTree {
+	CombineMode		mode;
+	GpPath*			path;
+	struct GpPathTree*	branch1;
+	struct GpPathTree*	branch2;
+} GpPathTree;
+
 void gdip_region_clear_tree (GpPathTree *tree) GDIP_INTERNAL;
-void gdip_region_copy_tree (GpPathTree *source, GpPathTree *dest) GDIP_INTERNAL;
+GpStatus gdip_region_copy_tree (GpPathTree *source, GpPathTree *dest) GDIP_INTERNAL;
 
 UINT gdip_region_get_tree_size (GpPathTree *tree) GDIP_INTERNAL;
 BOOL gdip_region_deserialize_tree (BYTE *data, int size, GpPathTree *tree) GDIP_INTERNAL;
