@@ -25,6 +25,7 @@
  */
 
 #include "solidbrush-private.h"
+#include "general-private.h"
 #include "graphics-private.h"
 
 static GpStatus gdip_solidfill_setup (GpGraphics *graphics, GpBrush *brush);
@@ -45,11 +46,11 @@ static void
 gdip_solidfill_init (GpSolidFill *brush)
 {
 	gdip_brush_init (&brush->base, &vtable);
-	brush->color = 0;
+	brush->color = 0x00000000;
 }
 
 static GpSolidFill*
-gdip_solidfill_new (void)
+gdip_solidfill_new ()
 {
 	GpSolidFill *result = (GpSolidFill *) GdipAlloc (sizeof (GpSolidFill));
 	
@@ -63,7 +64,6 @@ GpStatus
 gdip_solidfill_setup (GpGraphics *graphics, GpBrush *brush)
 {
 	GpSolidFill *solid;
-	int A, R, G, B;
 
 	if (!graphics || !brush)
 		return InvalidParameter;
@@ -74,26 +74,26 @@ gdip_solidfill_setup (GpGraphics *graphics, GpBrush *brush)
 	 * brush is changed.
 	 */
 	if (solid->base.changed) {
-		A = (solid->color & 0xFF000000) >> 24;
-		R = (solid->color & 0x00FF0000) >> 16;
-		G = (solid->color & 0x0000FF00) >> 8;
-		B = (solid->color & 0x000000FF);
+		BYTE a = (solid->color & 0xFF000000) >> 24;
+		BYTE r = (solid->color & 0x00FF0000) >> 16;
+		BYTE g = (solid->color & 0x0000FF00) >> 8;
+		BYTE b = (solid->color & 0x000000FF);
 
-		solid->A = (double) A / 255.0;
-		solid->R = (double) R / 255.0;
-		solid->G = (double) G / 255.0;
-		solid->B = (double) B / 255.0;
+		if (a == 0) {
+			solid->A = 0;
+			solid->R = 0;
+			solid->G = 0;
+			solid->B = 0;
+		}
+		else {
+			solid->A = (double) a / 255.0;
+			solid->R = (double) r / 255.0;
+			solid->G = (double) g / 255.0;
+			solid->B = (double) b / 255.0;
+		}
 	}
 
-	/*
-	 * Controls whether to use the alpha component in the color
-	 * or not.
-	 */
-	if (graphics->composite_mode == CompositingModeSourceOver)
-	  cairo_set_source_rgba (graphics->ct, solid->R, solid->G, solid->B, solid->A);
-	else
-	  cairo_set_source_rgb (graphics->ct, solid->R, solid->G, solid->B);
-	  
+	cairo_set_source_rgba (graphics->ct, solid->R, solid->G, solid->B, solid->A);
 	return Ok;
 }
 
@@ -103,28 +103,23 @@ gdip_solidfill_clone (GpBrush *brush, GpBrush **clonedBrush)
 	GpSolidFill *result;
 	GpSolidFill *solid;
 
-	/* the NULL checks for brush and clonedBrush are done by the caller, GdipCloneBrush */
-
-	result = (GpSolidFill *) GdipAlloc (sizeof (GpSolidFill));
+	result = gdip_solidfill_new ();
 	if (!result)
 		return OutOfMemory;
 
 	solid = (GpSolidFill *) brush;
 
 	result->base = solid->base;
-        result->color = solid->color;
+	result->color = solid->color;
 	result->base.changed = TRUE;
 
 	*clonedBrush = (GpBrush *) result;
-
 	return Ok;
 }
 
 GpStatus
 gdip_solidfill_destroy (GpBrush *brush)
 {
-	/* a. the NULL check for brush is done by the caller, GdipDeleteBrush */
-	/* b. brush itself is freed by the caller */
 	return Ok;
 }
 
@@ -132,14 +127,21 @@ gdip_solidfill_destroy (GpBrush *brush)
 GpStatus WINGDIPAPI
 GdipCreateSolidFill (ARGB color, GpSolidFill **brush)
 {
+	GpSolidFill *result;
+
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
 	if (!brush)
 		return InvalidParameter;
 
-	*brush = gdip_solidfill_new ();
-	if (!*brush)
+	result = gdip_solidfill_new ();
+	if (!result)
 		return OutOfMemory;
 
-	(*brush)->color = color;
+	result->color = color;
+	
+	*brush = result;
 	return Ok;
 }
 
@@ -149,10 +151,10 @@ GdipSetSolidFillColor (GpSolidFill *brush, ARGB color)
 	if (!brush)
 		return InvalidParameter;
 
-        brush->color = color;
+	brush->color = color;
 	brush->base.changed = TRUE;
 
-        return Ok;
+	return Ok;
 }
 
 GpStatus WINGDIPAPI
@@ -161,6 +163,6 @@ GdipGetSolidFillColor (GpSolidFill *brush, ARGB *color)
 	if (!brush || !color)
 		return InvalidParameter;
 
-        *color = brush->color;
-        return Ok;
+	*color = brush->color;
+	return Ok;
 }
